window.Gleam.initSearch([{"doc":"glimit","title":"glimit","content":"# glimit\n\n[![Package Version](https://img.shields.io/hexpm/v/glimit)](https://hex.pm/packages/glimit)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glimit/)\n[![test](https://github.com/nootr/glimit/actions/workflows/test.yml/badge.svg)](https://github.com/nootr/glimit/actions/workflows/test.yml)\n\nA simple, framework-agnostic, in-memory rate limiter for Gleam. 💫\n\n&gt; ⚠️  This library is still in development, use at your own risk.\n\n\n## Features\n\n* ✨ Simple and easy to use.\n* 📏 Rate limits based on any key (e.g. IP address, or user ID).\n* 🪣 Uses a distributed Token Bucket algorithm to rate limit requests.\n* 🗄️ No back-end service needed; stores rate limit stats in-memory.\n\n\n## Usage\n\nA very minimalistic example of how to use `glimit` would be the following snippet:\n\n```gleam\nimport glimit\n\nlet limiter =\n  glimit.new()\n  |&gt; glimit.per_second(2)\n  |&gt; glimit.identifier(fn(x) { x })\n  |&gt; glimit.on_limit_exceeded(fn(_) { &quot;Stop!&quot; })\n  |&gt; glimit.build\n\nlet func =\n  fn(_) { &quot;OK&quot; }\n  |&gt; glimit.apply(limiter)\n\nfunc(&quot;🚀&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;Stop!&quot;\nfunc(&quot;🚀&quot;) // &quot;OK&quot;\nfunc(&quot;🚀&quot;) // &quot;Stop!&quot;\n```\n\nA more practical example would be to use `glimit` to rate limit requests to a mist HTTP server:\n\n```gleam\nimport glimit\n\n\nfn handle_request(req: Request(Connection)) -&gt; Response(ResponseData) {\n  let index =\n    response.new(200)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  let not_found =\n    response.new(404)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n\n  case request.path_segments(req) {\n    [] -&gt; index\n    _ -&gt; not_found\n  }\n}\n\nfn get_identifier(req: Request(Connection)) -&gt; Result(String, String) {\n  req.body\n  |&gt; get_client_info\n  |&gt; result.map(fn(client_info: ConnectionInfo) {\n    client_info.ip_address |&gt; string.inspect\n  })\n  |&gt; result.unwrap(&quot;unknown IP address&quot;)\n}\n\npub fn main() {\n  let rate_limit_reached = fn(_req) -&gt; {\n    response.new(429)\n    |&gt; response.set_body(mist.Bytes(bytes_builder.new()))\n  }\n\n  let limiter =\n    glimit.new()\n    |&gt; glimit.per_second(10)\n    |&gt; glimit.burst_limit(100)\n    |&gt; glimit.identifier(get_identifier)\n    |&gt; glimit.on_limit_exceeded(rate_limit_reached)\n    |&gt; glimit.build\n\n  let assert Ok(_) =\n    handle_request\n    |&gt; glimit.apply(limiter)\n    |&gt; mist.new\n    |&gt; mist.port(8080)\n    |&gt; mist.start_http\n\n  process.sleep_forever()\n}\n```\n\n\n## Constraints\n\nWhile the in-memory rate limiter is simple and easy to use, it does have an important constraint: it is scoped to the BEAM VM cluster it runs in. This means that if your application is running across multiple BEAM VM clusters, the rate limiter will not be shared between them.\n\nThere are plans to add support for a centralized data store using Redis in the future.\n\n\n## Documentation\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glimit/glimit.html&gt;.\n\n\n## Contributing\n\nContributions like PR&#39;s, bug reports or suggestions are more than welcome! ♥️\n","url":"index.html"},{"doc":"glimit","title":"RateLimiter","content":"pub type RateLimiter(a, b, id) {\n  RateLimiter(\n    rate_limiter_registry: RateLimiterRegistryActor(id),\n    on_limit_exceeded: fn(a) -&gt; b,\n    identifier: fn(a) -&gt; id,\n  )\n}\n A rate limiter.\n\nRateLimiter(\n  rate_limiter_registry: RateLimiterRegistryActor(id),\n  on_limit_exceeded: fn(a) -&gt; b,\n  identifier: fn(a) -&gt; id,\n)\n\n\nSynonyms:\nglimit.RateLimiter\nglimit RateLimiter","url":"glimit.html#RateLimiter"},{"doc":"glimit","title":"RateLimiterBuilder","content":"pub type RateLimiterBuilder(a, b, id) {\n  RateLimiterBuilder(\n    per_second: Option(Int),\n    burst_limit: Option(Int),\n    identifier: Option(fn(a) -&gt; id),\n    on_limit_exceeded: Option(fn(a) -&gt; b),\n  )\n}\n A builder for configuring the rate limiter.\n\nRateLimiterBuilder(\n  per_second: Option(Int),\n  burst_limit: Option(Int),\n  identifier: Option(fn(a) -&gt; id),\n  on_limit_exceeded: Option(fn(a) -&gt; b),\n)\n\n\nSynonyms:\nglimit.RateLimiterBuilder\nglimit RateLimiterBuilder","url":"glimit.html#RateLimiterBuilder"},{"doc":"glimit","title":"apply","content":"pub fn apply(\n  func: fn(a) -&gt; b,\n  limiter: RateLimiter(a, b, c),\n) -&gt; fn(a) -&gt; b\n Apply the rate limiter to a request handler or function.\n\nSynonyms:\nglimit.apply\nglimit apply","url":"glimit.html#apply"},{"doc":"glimit","title":"build","content":"pub fn build(\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; RateLimiter(a, b, c)\n Build the rate limiter.\n\n Panics if the rate limiter registry cannot be started or if the `identifier`\n function or `on_limit_exceeded` function is missing.\n\n To handle errors instead of panicking, use `try_build`.\n\nSynonyms:\nglimit.build\nglimit build","url":"glimit.html#build"},{"doc":"glimit","title":"burst_limit","content":"pub fn burst_limit(\n  limiter: RateLimiterBuilder(a, b, c),\n  burst_limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the maximum number of available tokens.\n\n The maximum number of available tokens is the maximum number of requests that can be\n made in a single second. The default value is the same as the rate limit per second.\n\nSynonyms:\nglimit.burst_limit\nglimit burst_limit","url":"glimit.html#burst_limit"},{"doc":"glimit","title":"identifier","content":"pub fn identifier(\n  limiter: RateLimiterBuilder(a, b, c),\n  identifier: fn(a) -&gt; c,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the identifier function to be used to identify the rate limit.\n\nSynonyms:\nglimit.identifier\nglimit identifier","url":"glimit.html#identifier"},{"doc":"glimit","title":"new","content":"pub fn new() -&gt; RateLimiterBuilder(a, b, c)\n Create a new rate limiter builder.\n\nSynonyms:\nglimit.new\nglimit new","url":"glimit.html#new"},{"doc":"glimit","title":"on_limit_exceeded","content":"pub fn on_limit_exceeded(\n  limiter: RateLimiterBuilder(a, b, c),\n  on_limit_exceeded: fn(a) -&gt; b,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the handler to be called when the rate limit is reached.\n\nSynonyms:\nglimit.on_limit_exceeded\nglimit on_limit_exceeded","url":"glimit.html#on_limit_exceeded"},{"doc":"glimit","title":"per_second","content":"pub fn per_second(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per second.\n\n The value is not only used for the rate at which tokens are added to the bucket, but\n also for the maximum number of available tokens. To set a different value fo the\n maximum number of available tokens, use the `burst_limit` function.\n\nSynonyms:\nglimit.per_second\nglimit per_second","url":"glimit.html#per_second"},{"doc":"glimit","title":"try_build","content":"pub fn try_build(\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; Result(RateLimiter(a, b, c), String)\n Build the rate limiter, but return an error instead of panicking.\n\nSynonyms:\nglimit.try_build\nglimit try_build","url":"glimit.html#try_build"},{"doc":"glimit","title":"glimit","content":" This module provides a distributed rate limiter that can be used to limit the\n number of requests or function calls per second for a given identifier.\n\n A single actor is used to assign one rate limiter actor per identifier. The\n rate limiter actor then uses a Token Bucket algorithm to determine if a\n request or function call should be allowed to proceed. A separate process is\n polling the rate limiters to remove full buckets to reduce unnecessary memory\n usage.\n\n The rate limits are configured using the following two options:\n\n - `per_second`: The rate of new available tokens per second. Think of this\n   as the steady state rate limit.\n - `burst_limit`: The maximum number of available tokens. Think of this as\n   the burst rate limit. The default value is the `per_second` rate limit.\n\n The rate limiter can be applied to a function or handler using the `apply`\n function, which returns a new function that checks the rate limit before\n calling the original function.\n\n # Example\n\n ```gleam\n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_second(10)\n   |&gt; glimit.burst_limit(100)\n   |&gt; glimit.identifier(fn(request) { request.ip })\n   |&gt; glimit.on_limit_exceeded(fn(_request) { &quot;Rate limit reached&quot; })\n   |&gt; glimit.build()\n\n let handler =\n   fn(_request) { &quot;Hello, world!&quot; }\n   |&gt; glimit.apply(limiter)\n ```\n","url":"glimit.html"}]);